@@ -12,12 +12,14 @@
 #' @param facet_col Create horizontally arranged subplots for each level of the selected factor variable
 #' @param color Adds color to a scatter plot to generate a 'heat map'. For a line plot one line is created for each group and each is assigned a different color
 #' @param fill Display bar, distribution, and density plots by group, each with a different color. Also applied to surface plots to generate a 'heat map'
+#' @param size Numeric variable used to scale the size of scatter-plot points
 #' @param bins Number of bins used for a histogram (1 - 50)
 #' @param smooth Adjust the flexibility of the loess line for scatter plots
 #' @param fun Set the summary measure for line and bar plots when the X-variable is a factor (default is "mean"). Also used to plot an error bar in a scatter plot when the X-variable is a factor. Options are "mean" and/or "median"
 #' @param check Add a regression line ("line"), a loess line ("loess"), or jitter ("jitter") to a scatter plot
 #' @param axes Flip the axes in a plot ("flip") or apply a log transformation (base e) to the y-axis ("log_y") or the x-axis ("log_x")
 #' @param alpha Opacity for plot elements (0 to 1)
+#' @param ylim Set limit for y-axis
 #' @param data_filter Expression used to filter the dataset. This should be a string (e.g., "price > 10000")
 #' @param shiny Logical (TRUE, FALSE) to indicate if the function call originate inside a shiny app
 #' @param custom Logical (TRUE, FALSE) to indicate if ggplot object (or list of ggplot objects) should be returned. This opion can be used to customize plots (e.g., add a title, change x and y labels, etc.). See examples and \url{http://docs.ggplot2.org/} for options.
@@ -27,16 +29,20 @@
 #' @examples
 #' visualize("diamonds", "price:x", type = "dist")
 #' visualize("diamonds", "carat:x", yvar = "price", type = "scatter")
+#' \dontrun{
 #' visualize(dataset = "diamonds", yvar = "price", xvar = c("cut","clarity"),
 #'   type = "bar", fun = "median")
 #' visualize(dataset = "diamonds", yvar = "price", xvar = c("cut","clarity"),
 #'   type = "line", fun = "max")
+#' visualize(dataset = "diamonds", yvar = "price", xvar = "carat", type = "scatter",
+#'           size = "table", custom = TRUE) + scale_size(range=c(1,10), guide = "none")
 #' visualize(dataset = "diamonds", yvar = "price", xvar = "carat", type = "scatter", custom = TRUE) +
-#'   ggtitle("A scatterplot") + xlab("price in $")
+#'   labs(title = "A scatterplot", x = "price in $")
 #' visualize(dataset = "diamonds", xvar = "price:carat", custom = TRUE) %>%
-#'   {.[[1]] + ggtitle("A histogram") + xlab("price in $")}
+#'   gridExtra::grid.arrange(grobs = ., top = "Histograms", ncol = 2)
 #' visualize(dataset = "diamonds", xvar = "cut", yvar = "price", type = "bar",
-#'   facet_row = "cut", fill = "cut", custom = FALSE)
+#'   facet_row = "cut", fill = "cut")
+#' }
 #'
 #' @export
 visualize <- function(dataset, xvar,
@@ -48,12 +54,14 @@ visualize <- function(dataset, xvar,
                       facet_col = ".",
                       color = "none",
                       fill = "none",
+                      size = "none",
                       bins = 10,
                       smooth = 1,
                       fun = "mean",
                       check = "",
                       axes = "",
                       alpha = .5,
+                      ylim = "none",
                       data_filter = "",
                       shiny = FALSE,
                       custom = FALSE) {
@@ -66,6 +74,7 @@ visualize <- function(dataset, xvar,
   if (type != "scatter") {
     check %<>% sub("line","",.) %>% sub("loess","",.)
     fun <- fun[1]  # only scatter can deal with multiple functions
+    size <- "none"
   }
   if (!type %in% c("scatter","box")) check %<>% sub("jitter","",.)
 
@@ -98,15 +107,21 @@ visualize <- function(dataset, xvar,
     vars %<>% c(., facet_col)
     byvar <- if (is.null(byvar)) facet_col else unique(c(byvar, facet_col))
   }
+
+  if (facet_col != "." && facet_row == facet_col)
+    return("The same variable cannot be used for both Facet row and Facet column")
+
   if (fill != "none") {
     vars %<>% c(., fill)
     if (type == "bar")
       byvar <- if (is.null(byvar)) fill else unique(c(byvar, fill))
   }
+  if (size != "none") vars %<>% c(., size)
 
   ## so you can also pass-in a data.frame
   dat <- getdata(dataset, vars, filt = data_filter)
-  if (!is_string(dataset)) dataset <- "-----"
+  # if (!is_string(dataset)) dataset <- deparse(substitute(dataset))
+  if (!is_string(dataset)) dataset <- deparse(substitute(dataset)) %>% set_attr("df", TRUE)
 
   ## get class
   dc <- getclass(dat)
@@ -118,18 +133,18 @@ visualize <- function(dataset, xvar,
     xvar <- cn[which(fl[1] == cn):which(fl[2] == cn)]
   }
 
-  ## convertising factor variables if needed
+  ## converting character variables if needed
   isChar <- dc == "character"
   if (sum(isChar) > 0) {
     if (type == "density") {
-      dat[,isChar] <- select(dat, which(isChar)) %>% mutate_each(funs(as_numeric))
+      dat[,isChar] <- select(dat, which(isChar)) %>% mutate_all(funs(as_numeric))
       if ("character" %in% getclass(select(dat,which(isChar))))
-        return("Character variable(s) were not converted to numeric.\nTo use these variables in a plot convert them to numeric\nvariables (or factors) in the Data > Transform tab")
+        return("Character variable(s) were not converted to numeric for plotting.\nTo use these variables in a plot convert them to numeric\nvariables (or factors) in the Data > Transform tab")
     } else {
-      dat[,isChar] <- select(dat, which(isChar)) %>% mutate_each(funs(as_factor))
+      dat[,isChar] <- select(dat, which(isChar)) %>% mutate_all(funs(as_factor))
       nrlev <- sapply(dat, function(x) if (is.factor(x)) length(levels(x)) else 0)
-      if (max(nrlev) > 100)
-        return("Character variable(s) were not converted to factors.\nTo use these variable in a plot convert them to factors\n(or numeric variables) in the Data > Transform tab")
+      if (max(nrlev) > 500)
+        return("Character variable(s) were not converted to factors for plotting.\nTo use these variable in a plot convert them to factors\n(or numeric variables) in the Data > Transform tab")
     }
     ## in case something was changed, if not, this won't run
     dc <- getclass(dat)
@@ -146,11 +161,11 @@ visualize <- function(dataset, xvar,
   }
 
   ## 1 of first level of factor, else 0
+  # if (type == "bar" || type == "scatter" || type == "line") {
   if (type == "bar") {
-
     isFctY <- "factor" == dc & names(dc) %in% yvar
     if (sum(isFctY)) {
-      dat[,isFctY] <- select(dat, which(isFctY)) %>% mutate_each(funs(as.integer(. == levels(.)[1])))
+      dat[,isFctY] <- select(dat, which(isFctY)) %>% mutate_all(funs(as.integer(. == levels(.)[1])))
       dc[isFctY] <- "integer"
     }
   }
@@ -168,14 +183,14 @@ visualize <- function(dataset, xvar,
     if (any(!dc[xvar] %in% c("integer","numeric")))
       return("'Log X' is only meaningful for X-variables of type integer or numeric")
     to_log <- (dc[xvar] %in% c("integer","numeric")) %>% xvar[.]
-    dat[, to_log] <- select_(dat, .dots = to_log) %>% mutate_each(funs(log_trans))
+    dat[, to_log] <- select_(dat, .dots = to_log) %>% mutate_all(funs(log_trans))
   }
 
   if ("log_y" %in% axes) {
     if (any(!dc[yvar] %in% c("integer","numeric")))
       return("'Log Y' is only meaningful for Y-variables of type integer or numeric")
     to_log <- (dc[yvar] %in% c("integer","numeric")) %>% yvar[.]
-    dat[, to_log] <- select_(dat, .dots = to_log) %>% mutate_each(funs(log_trans))
+    dat[, to_log] <- select_(dat, .dots = to_log) %>% mutate_all(funs(log_trans))
   }
 
   ## combining Y-variables if needed
@@ -183,9 +198,10 @@ visualize <- function(dataset, xvar,
     if (any(xvar %in% yvar)) return("X-variables cannot be part of Y-variables when combining Y-variables")
     if (!is_empty(color, "none")) return("Cannot use Color when combining Y-variables")
     if (!is_empty(fill, "none")) return("Cannot use Fill when combining Y-variables")
+    if (!is_empty(size, "none")) return("Cannot use Size when combining Y-variables")
     if (facet_row %in% yvar || facet_col %in% yvar) return("Facet row or column variables cannot be part of\nY-variables when combining Y-variables")
 
-    dat <- gather_(dat, "yvar", "values", gather_cols = yvar)
+    dat <- gather_(dat, "yvar", "values", gather_cols = yvar, factor_key = TRUE)
     yvar <- "values"
     byvar <- if (is.null(byvar)) "yvar" else c("yvar", byvar)
     color <- fill <- "yvar"
@@ -199,7 +215,7 @@ visualize <- function(dataset, xvar,
     if (facet_row %in% xvar || facet_col %in% xvar) return("Facet row or column variables cannot be part of\nX-variables when combining Y-variables")
     if (any(!getclass(select_(dat, .dots = xvar)) %in% c("numeric","integer"))) return("Cannot combine plots for non-numeric variables")
 
-    dat <- gather_(dat, "xvar", "values", gather_cols = xvar)
+    dat <- gather_(dat, "xvar", "values", gather_cols = xvar, factor_key = TRUE)
     xvar <- "values"
     byvar <- if (is.null(byvar)) "xvar" else c("xvar", byvar)
     color <- fill <- "xvar"
@@ -217,10 +233,10 @@ visualize <- function(dataset, xvar,
         dc[i] <- "factor"
       }
 
-      hist_par <- list(alpha = alpha, position = "dodge")
+      hist_par <- list(alpha = alpha, position = "identity")
       plot_list[[i]] <- ggplot(dat, aes_string(x = i))
       if ("density" %in% axes && !"factor" %in% dc[i]) {
-        hist_par <- list(aes(y = ..density..), alpha = alpha, position = "dodge")
+        hist_par <- list(aes(y = ..density..), alpha = alpha, position = "identity")
         plot_list[[i]] <- plot_list[[i]] + geom_density(color = "blue", size = .5)
       }
       if ("factor" %in% dc[i]) {
@@ -323,7 +339,8 @@ visualize <- function(dataset, xvar,
         if (color == 'none') {
           if (dc[i] %in% c("factor","date")) {
             tbv <- if (is.null(byvar)) i else c(i, byvar)
-            tmp <- dat %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, j)) %>% summarise_each(make_funs(fun))
+            tmp <- dat %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, j)) %>% 
+              summarise_all(make_funs(fun))
             colnames(tmp)[ncol(tmp)] <- j
             plot_list[[itt]] <- ggplot(tmp, aes_string(x = i, y = j)) + geom_line(aes(group = 1))
             if (nrow(tmp) < 101) plot_list[[itt]] <- plot_list[[itt]] + geom_point()
@@ -333,7 +350,8 @@ visualize <- function(dataset, xvar,
         } else {
           if (dc[i] %in% c("factor","date")) {
             tbv <- if (is.null(byvar)) i else unique(c(i, byvar))
-            tmp <- dat %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, color, j)) %>% summarise_each(make_funs(fun))
+            tmp <- dat %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, color, j)) %>% 
+              summarise_all(make_funs(fun))
             colnames(tmp)[ncol(tmp)] <- j
             plot_list[[itt]] <- ggplot(tmp, aes_string(x = i, y = j, color = color, group = color)) + geom_line()
             if (nrow(tmp) < 101) plot_list[[itt]] <- plot_list[[itt]] + geom_point()
@@ -357,11 +375,15 @@ visualize <- function(dataset, xvar,
       if ("log_x" %in% axes) axes <- sub("log_x","",axes)
       for (j in yvar) {
         tbv <- if (is.null(byvar)) i else c(i, byvar)
-        tmp <- dat %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, j)) %>% summarise_each(make_funs(fun))
+        tmp <- dat %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, j)) %>% 
+          summarise_all(make_funs(fun))
         colnames(tmp)[ncol(tmp)] <- j
 
         if ("sort" %in% axes && facet_row == "." && facet_col == ".") {
-          tmp <- arrange_(ungroup(tmp), j)
+          if ("flip" %in% axes)
+            tmp <- arrange_(ungroup(tmp), j)
+          else
+            tmp <- arrange_(ungroup(tmp), paste0("desc(",j,")"))
           tmp[[i]] %<>% factor(., levels = unique(.))
         }
 
@@ -370,7 +392,7 @@ visualize <- function(dataset, xvar,
 
         if ("log_y" %in% axes) plot_list[[itt]] <- plot_list[[itt]] + ylab(paste("log", j))
 
-        if (dc[i] %in% c("factor","date") && nrow(tmp) < nrow(dat))
+        if (dc[i] %in% c("factor","integer","date") && nrow(tmp) < nrow(dat))
           plot_list[[itt]]$labels$y %<>% paste0(., " (", fun, ")")
 
         itt <- itt + 1
@@ -407,12 +429,21 @@ visualize <- function(dataset, xvar,
       plot_list[[i]] <- plot_list[[i]] + aes_string(color = color)
   }
 
-  ## adding fill
+  if (size != "none") {
+    for (i in 1:length(plot_list))
+      plot_list[[i]] <- plot_list[[i]] + aes_string(size = size)
+  }
+
   if (fill != "none") {
     for (i in 1:length(plot_list))
       plot_list[[i]] <- plot_list[[i]] + aes_string(fill = fill)
   }
 
+  if (ylim != "none" && is.numeric(ylim) && length(ylim) == 2) {
+    for (i in 1:length(plot_list))
+      plot_list[[i]] <- plot_list[[i]] + ylim(ylim[1], ylim[2])
+  }
+
   if ("jitter" %in% check) {
     for (i in 1:length(plot_list))
       plot_list[[i]] <- plot_list[[i]] +
@@ -441,6 +472,6 @@ visualize <- function(dataset, xvar,
   if (custom)
     if (length(plot_list) == 1) return(plot_list[[1]]) else return(plot_list)
 
-  sshhr( do.call(gridExtra::arrangeGrob, c(plot_list, list(ncol = min(length(plot_list), 2)))) ) %>%
-   {if (shiny) . else print(.)}
+  sshhr(gridExtra::grid.arrange(grobs = plot_list, ncol = min(length(plot_list), 2))) %>%
+    {if (shiny) . else print(.)}
 }
