@@ -64,114 +64,23 @@ output$ui_viz_xvar <- renderUI({
     multiple = TRUE, size = min(3, length(vars)), selectize = FALSE)
 })
 
-# output$ui_viz_comby <- renderUI({
-#   req(input$viz_yvar)
-#   if (length(input$viz_yvar) < 2) {
-#     # isolate({
-#     #   r_state[["viz_comby"]] <<- FALSE
-#     #   updateCheckboxInput(session, "viz_comby", value = FALSE)
-#     # })
-#     r_state[["viz_comby"]] <<- FALSE
-#     return()
-#   }
-#   checkboxInput("viz_comby", "Combine Y-variables in one plot",
-#     .state_init("viz_comby", FALSE))
-# })
-
-# output$ui_viz_combx <- renderUI({
-#   req(input$viz_xvar)
-#   if (length(input$viz_xvar) < 2) {
-#     # isolate({
-#       # r_state[["viz_combx"]] <<- FALSE
-#       # updateCheckboxInput(session, "viz_combx", value = FALSE)
-#     # })
-#     r_state[["viz_combx"]] <<- FALSE
-#     checkboxInput("viz_combx", "Combine X-variables in one plot", FALSE)
-#     # return()
-#   } else {
-#     checkboxInput("viz_combx", "Combine X-variables in one plot",
-#       state_init("viz_combx", FALSE))
-#   }
-# })
-
-# observeEvent(length(input$viz_yvar) < 2, {
-#   r_state[["viz_comby"]] <<- FALSE
-#   updateCheckboxInput(session, "viz_comby", value = FALSE)
-# })
-
 output$ui_viz_comby <- renderUI({
-  # if (length(input$viz_yvar) < 2) return(NULL)
-  # checkboxInput("viz_comby", "Combine Y-variables in one plot",
-  #   state_init("viz_comby", FALSE))
-
-  if (length(input$viz_yvar) > 1) {
-    checkboxInput("viz_comby", "Combine Y-variables in one plot",
-      state_init("viz_comby", FALSE))
-  } else {
-    return()
-  }
+  checkboxInput("viz_comby", "Combine Y-variables in one plot",
+    state_init("viz_comby", FALSE))
 })
 
-# observeEvent(!is.null(input$viz_xvar) && length(input$viz_xvar) < 2, {
-#   r_state[["viz_combx"]] <<- FALSE
-#   updateCheckboxInput(session, "viz_combx", value = FALSE)
-# })
-
-# observeEvent(length(input$viz_xvar) == 1, {
-#   r_state[["viz_combx"]] <<- FALSE
-#   updateCheckboxInput(session, "viz_combx", value = FALSE)
-# })
-
-# observeEvent(input$viz_xvar, {
-#   if (length(input$viz_xvar) < 2) {
-#     r_state[["viz_combx"]] <<- FALSE
-#     updateCheckboxInput(session, "viz_combx", value = FALSE)
-#   }
-# })
-
 output$ui_viz_combx <- renderUI({
-  # req(length(input$viz_xvar) > 1)
-  # if (!is.null(input$viz_xvar) && length(input$viz_xvar) < 2) return(NULL)
-  # if (!is.null(input$viz_xvar) && length(input$viz_xvar) < 2) return(NULL)
-  if (length(input$viz_xvar) > 1) {
-    checkboxInput("viz_combx", "Combine X-variables in one plot",
-      state_init("viz_combx", FALSE))
-  } else {
-    return()
-  }
+  checkboxInput("viz_combx", "Combine X-variables in one plot",
+    state_init("viz_combx", FALSE))
 })
 
-# observeEvent(input$viz_xvar < 2, {
-#   # if (length(input$viz_xvar) < 2) {
-    # r_state[["viz_combx"]] <<- FALSE
-    # updateCheckboxInput(session, "viz_combx", value = FALSE)
-  # }
-# })
+observeEvent(length(input$viz_xvar) < 2, {
+  updateCheckboxInput(session, "viz_combx", value = FALSE)
+})
 
-# observeEvent(input$viz_xvar < 2, {
-# observe({
-#   if (length(input$viz_xvar) < 2)
-#     updateCheckboxInput(session, "viz_combx", value = FALSE)
-# })
-
-# output$ui_viz_combx <- renderUI({
-#   # req(input$viz_xvar > 1) {
-#   # req(input$viz_xvar)
-#   if (length(input$viz_xvar) > 1) {
-#     checkboxInput("viz_combx", "Combine X-variables in one plot", FALSE)
-#     # checkboxInput("viz_combx", "Combine X-variables in one plot",
-#     #   .state_init("viz_combx", FALSE))
-#   } else {
-#     # updateCheckboxInput(session, "viz_combx", value = FALSE)
-#     return()
-#   }
-
-#   # } else {
-#   #   r_state[["viz_combx"]] <<- FALSE
-#   #   isolate(updateCheckboxInput(session, "viz_combx", value = FALSE))
-#   #   return()
-#   # }
-# })
+observeEvent(length(input$viz_yvar) < 2, {
+  updateCheckboxInput(session, "viz_comby", value = FALSE)
+})
 
 observeEvent(input$viz_type, {
   if (input$viz_type %in% c("dist", "density")) {
@@ -181,22 +90,6 @@ observeEvent(input$viz_type, {
   }
 })
 
-# observeEvent(input$viz_combx, {
-#   if (input$viz_combx) {
-#   # if (input$viz_combx && length(input$viz_xvar) > 1) {
-#     updateCheckboxInput(session, "viz_color", value = "none")
-#     updateCheckboxInput(session, "viz_fill", value = "none")
-#   }
-# })
-
-# observeEvent(input$viz_comby, {
-#   if (input$viz_comby) {
-#   # if (input$viz_combx && input$viz_yvar > 1) {
-#     updateCheckboxInput(session, "viz_color", value = "none")
-#     updateCheckboxInput(session, "viz_fill", value = "none")
-#   }
-# })
-
 observeEvent(input$viz_check, {
   if (!"loess" %in% input$viz_check && input$viz_smooth != 1)
     updateSliderInput(session, "viz_smooth", value = 1)
@@ -223,24 +116,25 @@ output$ui_viz_color <- renderUI({
   else
     vars <- c("None" = "none", varnames())
 
-  if (isTRUE(input$viz_comby) && length(input$viz_yvar) > 1) {
-    vars <- c("None" = "none")
-    selectizeInput("viz_color", "Color", vars, multiple = FALSE, selected = "none")
-  } else {
-    selectizeInput("viz_color", "Color", vars, multiple = FALSE,
-      selected = state_single("viz_color", vars, init = "none"))
-  }
+  if (isTRUE(input$viz_comby) && length(input$viz_yvar) > 1) vars <- c("None" = "none")
+  selectizeInput("viz_color", "Color", vars, multiple = FALSE,
+    selected = state_single("viz_color", vars, init = "none"))
 })
 
 output$ui_viz_fill <- renderUI({
-  if (isTRUE(input$viz_combx) && length(input$viz_xvar) > 1) {
-    vars <- c("None" = "none")
-    selectizeInput("viz_fill", "Fill", vars, multiple = FALSE, selected = "none")
-  } else {
-    vars <- c("None" = "none", groupable_vars())
-    selectizeInput("viz_fill", "Fill", vars, multiple = FALSE,
-      selected = state_single("viz_fill", vars, init = "none"))
-  }
+  vars <- c("None" = "none", groupable_vars())
+  if (isTRUE(input$viz_combx) && length(input$viz_xvar) > 1) vars <- vars[1]
+  selectizeInput("viz_fill", "Fill", vars, multiple = FALSE,
+    selected = state_single("viz_fill", vars, init = "none"))
+})
+
+output$ui_viz_size <- renderUI({
+  req(input$viz_type)
+  isNum <- .getclass() %in% c("integer", "numeric")
+  vars <- c("None" = "none", varnames()[isNum])
+  if (isTRUE(input$viz_comby) && length(input$viz_yvar) > 1) vars <- c("None" = "none")
+  selectizeInput("viz_size", "Size", vars, multiple = FALSE,
+    selected = state_single("viz_size", vars, init = "none"))
 })
 
 output$ui_viz_axes <- renderUI({
@@ -292,11 +186,15 @@ output$ui_Visualize <- renderUI({
       uiOutput("ui_viz_type"),
       conditionalPanel(condition = "input.viz_type != 'dist' & input.viz_type != 'density'",
         uiOutput("ui_viz_yvar"),
-        uiOutput("ui_viz_comby")
+        conditionalPanel("input.viz_yvar != undefined && input.viz_yvar != null && input.viz_yvar.length > 1",
+          uiOutput("ui_viz_comby")
+        )
       ),
       uiOutput("ui_viz_xvar"),
       conditionalPanel("input.viz_type == 'dist' | input.viz_type == 'density'",
-        uiOutput("ui_viz_combx")
+        conditionalPanel("input.viz_xvar != undefined && input.viz_xvar != null && input.viz_xvar.length > 1",
+          uiOutput("ui_viz_combx")
+        )
       ),
       uiOutput("ui_viz_facet_row"),
       uiOutput("ui_viz_facet_col"),
@@ -311,6 +209,9 @@ output$ui_Visualize <- renderUI({
                                     input.viz_type == 'box'",
         uiOutput("ui_viz_color")
       ),
+      conditionalPanel(condition = "input.viz_type == 'scatter'",
+        uiOutput("ui_viz_size")
+      ),
       conditionalPanel(condition = "input.viz_type == 'scatter' |
                                     input.viz_type == 'line' |
                                     input.viz_type == 'surface' |
@@ -377,14 +278,16 @@ output$visualize <- renderPlot({
     )
   }
 
-  .visualize() %>% { if (is.character(.)) {
-      plot(x = 1, type = 'n', main = paste0("\n",.), axes = FALSE, xlab = "", ylab = "")
-    } else if (is.null(.)) {
-      return(invisible())
-    } else {
-      withProgress(message = 'Making plot', value = 0.5, print(.))
+  withProgress(message = 'Making plot', value = 1, {
+    .visualize() %>% { if (is.character(.)) {
+        plot(x = 1, type = 'n', main = paste0("\n",.), axes = FALSE, xlab = "", ylab = "")
+      } else if (is.null(.)) {
+        return(invisible())
+      } else {
+        print(.)
+      }
     }
-  }
+  })
 }, width = viz_plot_width, height = viz_plot_height)
 
 .visualize <- reactive({
@@ -403,28 +306,11 @@ output$visualize <- renderPlot({
   if (input$viz_type %in% c("dist", "density")) {
     if (isTRUE(input$viz_comby)) return()
     if (length(input$viz_xvar) > 1 && is.null(input$viz_combx)) return()
-    # if (isTRUE(input$viz_combx)) {
-    #   if (!is_empty(input$viz_color,"none")) return()
-    #   if (!is_empty(input$viz_fill,"none")) return()
-    # }
   } else {
     if (isTRUE(input$viz_combx)) return()
     if (length(input$viz_yvar) > 1 && is.null(input$viz_comby)) return()
-    # if (isTRUE(input$viz_comby)) {
-    #   if (!is_empty(input$viz_color,"none")) return()
-    #   if (!is_empty(input$viz_fill,"none")) return()
-    # }
   }
 
-  # isolate({
-  #   print("-----")
-  #   print(length(input$viz_xvar))
-  #   print(isTRUE(input$viz_combx))
-  #   print(input$viz_combx)
-  #   print("-----")
-  # })
-
-  ## still doesn't stop flickering!
   req(!is.null(input$viz_color) || !is.null(input$viz_fill))
 
   viz_inputs() %>% { .$shiny <- TRUE; . } %>% do.call(visualize, .)
@@ -440,11 +326,7 @@ observeEvent(input$visualize_report, {
   if (!input$viz_type %in% c("scatter", "box") &&
       "jitter" %in% input$viz_check) vi$check <- setdiff(vi$check, "jitter")
 
-  if (isTRUE(input$viz_combx) && length(input$viz_xvar) < 2)
-      vi$combx <- FALSE
-
-  if (isTRUE(input$viz_comby) && length(input$viz_yvar) < 2)
-      vi$comby <- FALSE
+  if (!input$viz_type %in% "scatter") vi$size <- "none"
 
   inp_main <- clean_args(vi, viz_args)
   inp_main[["custom"]] <- FALSE
